@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,11 +26,9 @@
 /*! \brief Discard zone in zone database. */
 static void discard_zone(zone_t *zone, bool abort_txn)
 {
-	const knot_dname_t *unused = NULL;
-
 	// Don't flush if removed zone (no previous configuration available).
 	if (conf_rawid_exists(conf(), C_ZONE, zone->name, knot_dname_size(zone->name)) ||
-	    catalog_get_zone(conf()->catalog, zone->name, &unused) == KNOT_EOK) {
+	    catalog_has_member(conf()->catalog, zone->name)) {
 		uint32_t journal_serial, zone_serial = zone_contents_serial(zone->contents);
 		bool exists;
 
@@ -119,6 +117,24 @@ zone_t *knot_zonedb_find(knot_zonedb_t *db, const knot_dname_t *zone_name)
 	return *val;
 }
 
+zone_t **knot_zonedb_find_ptr(knot_zonedb_t *db, const knot_dname_t *zone_name)
+{
+	if (db == NULL) {
+		return NULL;
+	}
+
+	knot_dname_storage_t lf_storage;
+	uint8_t *lf = knot_dname_lf(zone_name, lf_storage);
+	assert(lf);
+
+	trie_val_t *val = trie_get_try(db->trie, lf + 1, *lf);
+	if (val == NULL) {
+		return NULL;
+	}
+
+	return (zone_t **)val;
+}
+
 zone_t *knot_zonedb_find_suffix(knot_zonedb_t *db, const knot_dname_t *zone_name)
 {
 	if (db == NULL || zone_name == NULL) {
